package com.tzld.piaoquan.growth.common.service.Impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.growth.common.component.HttpPoolClient;
import com.tzld.piaoquan.growth.common.dao.mapper.StaffMapper;
import com.tzld.piaoquan.growth.common.dao.mapper.StaffWithUserMapper;
import com.tzld.piaoquan.growth.common.dao.mapper.WeComUserMapper;
import com.tzld.piaoquan.growth.common.model.po.*;
import com.tzld.piaoquan.growth.common.service.WeComAccessTokenService;
import com.tzld.piaoquan.growth.common.service.WeComUserService;
import com.tzld.piaoquan.growth.common.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.tzld.piaoquan.growth.common.common.constant.WeComConstant.GET_WE_COM_EXTERNAL_CONTACT_GET;

@Slf4j
@Service
public class WeComUserServiceImpl implements WeComUserService {
    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private WeComAccessTokenService weComAccessTokenService;

    @Autowired
    private WeComUserMapper weComUserMapper;

    @Autowired
    private StaffWithUserMapper staffWithUserMapper;

    @Autowired
    private StaffMapper staffMapper;

    @Override
    public void addStaffWithUser(String externalUserId, String carrierId, Long corpId) {
        if (StringUtils.isEmpty(externalUserId) || StringUtils.isEmpty(carrierId)) {
            return;
        }
        Staff staff = getStaff(carrierId);
        insertStaffWithUser(externalUserId, staff, corpId);
    }

    @Override
    public void insertStaffWithUser(String externalUserId, Staff staff, Long corpId) {
        try {
            WeComUserExample weComUserExample = new WeComUserExample();
            weComUserExample.createCriteria().andExternalUserIdEqualTo(externalUserId);
            List<WeComUser> weComUserList = weComUserMapper.selectByExample(weComUserExample);
            if (CollectionUtils.isEmpty(weComUserList)) {
                JSONObject userDetail = getUserDetail(externalUserId, corpId);
                if (userDetail == null) {
                    return;
                }
                JSONObject externalContact = userDetail.getJSONObject("external_contact");
                JSONArray followUserList = userDetail.getJSONArray("follow_user");
                Long createAt = null;
                List<String> staffUserIdList = new ArrayList<>();
                for (int i = 0; i < followUserList.size(); i++) {
                    JSONObject followUser = followUserList.getJSONObject(i);
                    if (createAt == null) {
                        createAt = followUser.getLong("createtime");
                    } else {
                        createAt = Math.min(createAt, followUser.getLong("createtime"));
                    }
                    String userId = followUser.getString("userid");
                    if (StringUtils.isNotEmpty(userId)) {
                        staffUserIdList.add(userId);
                    }
                }
                String name = externalContact.getString("name");
                String unionId = externalContact.getString("unionid");
                String avatar = externalContact.getString("avatar");
                Integer type = externalContact.getInteger("type");
                Integer gender = externalContact.getInteger("gender");
                WeComUser weComUser = new WeComUser();
                weComUser.setExternalUserId(externalUserId);
                weComUser.setCreatedAt(createAt);
                weComUser.setName(name);
                weComUser.setUnionId(unionId);
                weComUser.setAvatar(avatar);
                weComUser.setType(type);
                weComUser.setGender(gender);
                weComUser.setCorpId(corpId);
                weComUserMapper.insert(weComUser);
                Long userId = weComUser.getId();
                if (userId == null) {
                    log.error("insertStaffWithUserJob insert weComUser error weComUser={}", weComUser);
                    return;
                }
                if (CollectionUtils.isEmpty(staffUserIdList)) {
                    return;
                }
                for (String staffUserId : staffUserIdList) {
                    StaffExample staffExample = new StaffExample();
                    staffExample.createCriteria().andCarrierIdEqualTo(staffUserId);
                    List<Staff> staffList1 = staffMapper.selectByExample(staffExample);
                    if (CollectionUtils.isEmpty(staffList1)) {
                        continue;
                    }
                    Staff staff1 = staffList1.get(0);
                    Long staffId = staff1.getId();
                    StaffWithUser staffWithUser = new StaffWithUser();
                    staffWithUser.setStaffId(staffId);
                    staffWithUser.setUserId(userId);
                    staffWithUserMapper.insert(staffWithUser);
                }
            } else {
                WeComUser weComUser = weComUserList.get(0);
                StaffWithUserExample staffWithUserExample = new StaffWithUserExample();
                staffWithUserExample.createCriteria().andUserIdEqualTo(weComUser.getId()).andStaffIdEqualTo(staff.getId());
                List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(staffWithUserExample);
                if (CollectionUtils.isEmpty(staffWithUserList)) {
                    StaffWithUser staffWithUser = new StaffWithUser();
                    staffWithUser.setStaffId(staff.getId());
                    staffWithUser.setUserId(weComUser.getId());
                    staffWithUserMapper.insert(staffWithUser);
                } else {
                    StaffWithUser staffWithUser = staffWithUserList.get(0);
                    if (staffWithUser.getIsDelete() == 1) {
                        staffWithUser.setIsDelete(0);
                        staffWithUserMapper.updateByPrimaryKeySelective(staffWithUser);
                    }
                }
            }
        } catch (Exception e) {
            LarkRobotUtil.sendMessage("insertUser error externalUserId=" + externalUserId + e);
            log.error("insertUser error", e);
        }
    }

    @Override
    public void delStaffWithUser(String externalUserId, String carrierId, Long deleteTime) {
        if (StringUtils.isEmpty(externalUserId) || StringUtils.isEmpty(carrierId)) {
            return;
        }
        Staff staff = getStaff(carrierId);
        if (staff == null) {
            return;
        }
        WeComUser weComUser = getUser(externalUserId);
        if (weComUser == null) {
            return;
        }
        StaffWithUserExample example = new StaffWithUserExample();
        example.createCriteria().andStaffIdEqualTo(staff.getId()).andUserIdEqualTo(weComUser.getId());
        List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(staffWithUserList) || staffWithUserList.get(0).getIsDelete() == 1) {
            return;
        }
        StaffWithUser staffWithUser = staffWithUserList.get(0);
        staffWithUser.setDeleteTime(new Date());
        staffWithUser.setIsDelete(1);
        staffWithUserMapper.updateByPrimaryKeySelective(staffWithUser);
    }

    @Override
    public void delStaffWithUser(Long userId, Long staffId, Long deleteTime) {
        if (userId == null || staffId == null) {
            return;
        }
        StaffWithUserExample example = new StaffWithUserExample();
        example.createCriteria().andStaffIdEqualTo(staffId).andUserIdEqualTo(userId);
        List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(staffWithUserList) || staffWithUserList.get(0).getIsDelete() == 1) {
            return;
        }
        StaffWithUser staffWithUser = staffWithUserList.get(0);
        if (deleteTime != null) {
            staffWithUser.setDeleteTime(new Date(deleteTime));
        }
        staffWithUser.setIsDelete(1);
        staffWithUserMapper.updateByPrimaryKeySelective(staffWithUser);
    }


    private Staff getStaff(String carrierId) {
        StaffExample example = new StaffExample();
        example.createCriteria().andCarrierIdEqualTo(carrierId);
        List<Staff> staffList = staffMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(staffList)) {
            log.error("getStaff staff empty carrierId={}", carrierId);
            return null;
        }
        return staffList.get(0);
    }

    private WeComUser getUser(String externalUserId) {
        WeComUserExample weComUserExample = new WeComUserExample();
        weComUserExample.createCriteria().andExternalUserIdEqualTo(externalUserId);
        List<WeComUser> weComUserList = weComUserMapper.selectByExample(weComUserExample);
        if (CollectionUtils.isEmpty(weComUserList)) {
            log.error("getUser user empty externalUserId={}", externalUserId);
            return null;
        }
        return weComUserList.get(0);
    }


    public JSONObject getUserDetail(String externalUserId, Long corpId) throws IOException {
        String weComAccessToken = weComAccessTokenService.getWeComAccessToken(corpId);
        String url = String.format(GET_WE_COM_EXTERNAL_CONTACT_GET + "?access_token=%s&external_userid=%s", weComAccessToken, externalUserId);
        String res = httpPoolClient.get(url);
        JSONObject jsonObject = JSONObject.parseObject(res);
        Integer errcode = jsonObject.getInteger("errcode");
        if (errcode == 0) {
            return jsonObject;
        }
        return null;
    }
}
